<?php

namespace Drupal\performance_budget\Plugin\CaptureResponse;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;
use Drupal\performance_budget\Helper\KpiHelperInterface;
use Drupal\web_page_archive\Plugin\CaptureResponse\UriCaptureResponse;
use Drupal\web_page_archive\Entity\WebPageArchiveRunInterface;

/**
 * Web page test capture response.
 */
class WebPageTestCaptureResponse extends UriCaptureResponse {

  /**
   * KPI Storage.
   *
   * @var Drupal\Core\Entity\EntityStorageInterface
   */
  protected $kpiStorage;

  /**
   * WPA Run Storage.
   *
   * @var Drupal\Core\Entity\EntityStorageInterface
   */
  protected $runStorage;

  /**
   * KPI helper service.
   *
   * @var Drupal\performance_budget\Helper\KpiHelper
   */
  protected $kpiHelper;

  /**
   * {@inheritdoc}
   */
  public static function getId() {
    return 'pb_wpt_capture_response';
  }

  /**
   * Retrieves the KPI storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The wpt_kpi entity storage.
   */
  protected function getKpiStorage() {
    if (!isset($this->kpiStorage)) {
      $this->kpiStorage = \Drupal::entityTypeManager()->getStorage('wpt_kpi');
    }
    return $this->kpiStorage;
  }

  /**
   * Sets the KPI storage.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The wpt_kpi entity storage.
   */
  public function setKpiStorage(EntityStorageInterface $storage) {
    $this->kpiStorage = $storage;
    return $this;
  }

  /**
   * Retrieves the web page archive run storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The web_page_archive_run entity storage.
   */
  protected function getRunStorage() {
    if (!isset($this->runStorage)) {
      $this->runStorage = \Drupal::entityTypeManager()->getStorage('web_page_archive_run');
    }
    return $this->runStorage;
  }

  /**
   * Sets the web page archive run storage.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The web_page_archive_run entity storage.
   */
  public function setRunStorage(EntityStorageInterface $storage) {
    $this->runStorage = $storage;
    return $this;
  }

  /**
   * Retrieves the KPI helper service.
   *
   * @return \Drupal\performance_budget\Helper\KpiHelperInterface
   *   The kpi helper service.
   */
  protected function getKpiHelper() {
    if (!isset($this->kpiHelper)) {
      $this->kpiHelper = \Drupal::service('performance_budget.helper.kpi');
    }
    return $this->kpiHelper;
  }

  /**
   * Sets the KPI helper service.
   *
   * @param \Drupal\performance_budget\Helper\KpiHelperInterface $kpi_helper
   *   The kpi helper service.
   */
  public function setKpiHelper(KpiHelperInterface $kpi_helper) {
    $this->kpiHelper = $kpi_helper;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function renderable(array $options = []) {
    $contents = $this->retrieveFileContents();
    $run = $this->getRunStorage()->loadRevision($options['vid']);
    $route_params = [
      'web_page_archive_run_revision' => $options['vid'],
      'delta' => $options['delta'],
    ];
    $render = [
      '#theme' => 'pb-wpt-preview',
      '#url' => $this->captureUrl,
      '#from' => isset($contents['from']) ? strip_tags($contents['from']) : $this->t('Unknown')->render(),
      '#kpis' => $this->parseKpis($run, $contents),
    ];

    // TODO: Summary links are only available for 30 days, so we may want
    // a better solution here.
    if (!empty($contents['summary'])) {
      $render['#view_button'] = [
        '#type' => 'link',
        '#url' => Url::fromUri($contents['summary']),
        '#title' => $this->t('View Detailed Report')->render(),
        '#attributes' => [
          'class' => ['button'],
          'target' => '_blank',
          'rel' => ['noopener', 'noreferrer'],
        ],
      ];
    }

    return $render;
  }

  /**
   * Parses content array for key performance indicators.
   *
   * @param \Drupal\web_page_archive\Entity\WebPageArchiveRunInterface $run
   *   Web page archive run entity.
   * @param array $content
   *   Content array generated by $this->retrieveFileContents().
   *
   * @return array
   *   List of KPI results.
   */
  public function parseKpis(WebPageArchiveRunInterface $run, array $content = NULL) {
    $ret = [];
    if (!isset($content)) {
      $content = $this->retrieveFileContents();
    }
    $capture_utilities = $run->getConfigEntity()->getCaptureUtilities()->getConfiguration();
    $kpi_group_ids = [];
    foreach ($capture_utilities as $capture_utility) {
      if ($capture_utility['id'] == 'pb_wpt_capture') {
        foreach ($capture_utility['data']['kpi_groups'] as $group) {
          if (!empty($group['target_id'])) {
            $kpi_group_ids[] = $group['target_id'];
          }
        }
      }
    }

    $kpi_groups = $this->getKpiStorage()->loadMultiple($kpi_group_ids);
    if (empty($kpi_groups)) {
      throw new \Exception('Missing KPI Group');
    }

    $kpi_map = $this->getKpiHelper()->getKpiMap();
    foreach ($kpi_groups as $kpi_group) {
      $kpis = $kpi_group->getKpis();
      foreach ($kpis as $average => $average_details) {
        foreach ($average_details as $view => $view_details) {
          foreach ($view_details as $kpi => $is_set) {
            $formatted_average = $this->getKpiHelper()->getFormattedAverageValue($average);
            $formatted_view = $this->getKpiHelper()->getFormattedViewValue($view);
            if (isset($content[$average][$view][$kpi])) {
              $ret[$kpi_group->label()][$formatted_average][$formatted_view][$kpi_map[$kpi]['title']->render()] = $this->getKpiHelper()->getFormattedKpiValue($kpi, $content[$average][$view][$kpi]);
            }
          }
        }
      }
    }
    return $ret;
  }

  /**
   * Retrieves file contents.
   */
  protected function retrieveFileContents() {
    if (!empty($this->content) && file_exists($this->content)) {
      return Json::decode(file_get_contents($this->content));
    }
    return [];
  }

}
